@@ -8,8 +8,8 @@
     Tuple, Dict, List, TypeVar
 """
 
-from typing import Union, Optional, TypeVar, _GenericAlias, \
-    _SpecialForm, _VariadicGenericAlias, Callable
+from typing import Union, Optional, Callable
+import typing
 from functools import wraps
 from tysig.tyerrors import DEFAULT_ERROR, SIG_TYPE_ERROR, ARGS_ERROR, \
     UNEXP_ERROR
@@ -20,26 +20,33 @@ class TySig(object):
     @staticmethod
     def getattr_name(invar) -> Optional[str]:
         """
-        gets name of input object
+        gets name of type of invar
+        i.e. gets __name__, or _name, or __origin__ of input object
 
         :param invar: object we are checking name of
         :return: name if exists otherwise None
         """
+        ftype = None
         try:
-            return invar.__getattribute__('_name')
-        except AttributeError:
+            ftype = invar.__getattribute__('_name')
+        except (AttributeError, TypeError):
             try:
-                return invar.__getattribute__('__name__')
-            except AttributeError:
-                return None
+                ftype = invar.__getattribute__('__name__')
+            except (AttributeError, TypeError):
+                pass
+        if ftype is None:
+            try:
+                ftype = str(invar.__getattribute__('__origin__'))
+                if "typing." in ftype:
+                    ftype = ftype.split('.')[1]
+            except (AttributeError, TypeError):
+                pass
+        return ftype
 
     @staticmethod
     def is_type(
             var: object,
-            check_type: Union[
-                type, TypeVar, _GenericAlias,
-                _SpecialForm, _VariadicGenericAlias
-            ]
+            check_type
     ) -> bool:
         """
         recursively checks if var is of type check_type
@@ -99,10 +106,9 @@ class TySig(object):
         :param vtype: we are checking the type of this type
         :return: True if is a typing type, else False
         """
-        ret = isinstance(vtype, type) or isinstance(vtype, _GenericAlias)
-        ret = ret or isinstance(vtype, _VariadicGenericAlias)
-        ret = ret or isinstance(vtype, _SpecialForm)
-        return ret
+        type_name = TySig.getattr_name(vtype)
+        typing_types = [x for x in typing.__dict__['__all__'] if x[0].isupper()]
+        return type_name in typing_types
 
     @staticmethod
     def signature(classobj: bool = False, **in_vars_types):
@@ -121,13 +127,15 @@ class TySig(object):
         def inner(fun: Callable):
             @wraps(fun)
             def sub(*_in_args, **in_kwargs):
+                # initialise vars_types (signature params), kwargs, args
                 in_args = _in_args[1:] if classobj else _in_args
                 istype = TySig.is_type
                 vars_types = in_vars_types.copy()
                 kwargs = in_kwargs.copy()
                 args = [x for x in in_args]
+                _NO_DEF = "__NO_DEF__"
 
-                # check default arguments which are of type tuple e.g. (2, int)
+                # check default types which are of type tuple e.g. (2, int)
                 defmap = dict()
                 for vname, vdeftype in vars_types.items():
                     if isinstance(vdeftype, tuple):
@@ -139,9 +147,12 @@ class TySig(object):
                         else:
                             defmap[vname] = vdef
 
+                # function to get the type and default value
+                # _NO_DEF represents no default value
                 def get_def_type(in_vdeftype) -> tuple:
-                    if TySig.is_typing_type(in_vdeftype):
-                        in_vdef, in_vtype = None, in_vdeftype
+                    if TySig.is_typing_type(in_vdeftype) or \
+                            isinstance(in_vdeftype, type):
+                        in_vdef, in_vtype = _NO_DEF, in_vdeftype
                     elif isinstance(in_vdeftype, tuple):
                         in_vdef, in_vtype = in_vdeftype
                     else:
@@ -149,7 +160,8 @@ class TySig(object):
                             SIG_TYPE_ERROR.format(type(in_vdeftype)))
                     return in_vdef, in_vtype
 
-                # check kwargs
+                # check types in kwargs
+                # if types are good then pop from kwargs
                 _kwargs = kwargs.copy()
                 for kw_name, kw_val in _kwargs.items():
                     vdeftype = vars_types.get(kw_name)
@@ -157,6 +169,8 @@ class TySig(object):
                         raise TypeError(
                             f"Unexpected variable found: '{kw_name}'")
 
+                    # we don't care about default values here (handled below)
+                    # we are checking if kwarg types are correct
                     _, vtype = get_def_type(vdeftype)
                     if not istype(kw_val, vtype):
                         raise TypeError(ARGS_ERROR.format(kw_name, vtype,
@@ -164,19 +178,23 @@ class TySig(object):
                     kwargs.pop(kw_name)
                     vars_types.pop(kw_name)
 
-                # check args
+                # assign values from args to kwargs
+                # go through all signature params and compare against args
                 for vname, vdeftype in vars_types.items():
                     vdef, vtype = get_def_type(vdeftype)
                     selected_arg = False
                     idx = -1
+                    # compare signature type against types of args
                     for idx, arg in enumerate(args):
+                        # if type matches, assign value or default
                         if not istype(arg, vtype):
-                            if vdef is None:
+                            if vdef == _NO_DEF:
                                 raise TypeError(
                                     ARGS_ERROR.format(vname, vtype, type(arg)))
-                            else:
+                            else:  # assign default
                                 kwargs[vname] = vdef
                                 break
+                        # type matches, assign value in kwargs
                         else:
                             kwargs[vname] = arg
                             selected_arg = True
@@ -197,7 +215,7 @@ class TySig(object):
                 # if running in a class then pass through self class object
                 if classobj:
                     return fun(_in_args[0], **kwargs)
-                else:  # otherwise: run function normally passing through all set args
+                else:  # ow: run function normally passing through all set args
                     return fun(**kwargs)
 
             return sub
